/* dg_cprog
 * Call progress test utility
 *
 * Copyright (c) 2006 Digivoice Eletronica
 * Author: Iruata Souza
 */

#include <linux/limits.h>

#include <stdio.h>
#include <stdlib.h>
#include <err.h>
#include <unistd.h>

#include <vlibdef.h>
#include <voicerlib.h>
#include <call_progress.h>

dg_event_data_structure event_context;
uint dtone, answered;

void
sighandler(int sig)
{
	int ret;
	
	/* force clean exit */
	switch (sig) {
	case SIGSTOP:
	case SIGQUIT:
	case SIGKILL:
	case SIGINT:
		printf("Turning off voicerlib...\n");
		ret = dg_ShutdownVoicerlib();
		if (ret)
			err(ret, "dg_ShutdownVoicerLib()");
		exit(0);
	}
}

void
event_handler(void *v)
{
	short ret;
	dg_event_data_structure *event;
	
	event = (dg_event_data_structure *)v;
	
	switch (event->command) {
		case EV_AFTERDIAL:
			ret = dg_EnableAnswerDetection(event->port);
			if (ret)
				err(1, "dg_EnableAnswerDetection(): %x", ret);
			
			ret = dg_EnableCallProgress(event->port, CP_ENABLE_ALL);
			if (ret)
				err(1, "dg_EnableCallProgress(): %x", ret);
		
			break;
		case EV_ANSWERED:
			ret = dg_DisableAnswerDetection(event->port);
			if (ret)
				err(1, "dg_DisableAnswerDetection(): %x", ret);

			ret = dg_EnableCallProgress(event->port, CP_ENABLE_BUSY_OR_FAX);
			if (ret)
				err(1, "dg_EnableCallProgress(): %x", ret);
			
			printf("Remote end answered.\n");
			answered = 1; 
			break;
		case EV_CALLING:
			printf(".");
			break;
		case EV_DIALTONE:
			if (!dtone) {
				printf("Deteced dialtone.\n");
				dtone = 1;
			}
			break;
		case EV_RINGS:
			printf("ring\n");	
			break;
		case EV_BUSY:
			printf("Line busy.\n");
			answered = 1;
			break;
	}
}

int 
main(int argc, char *argv[])
{
	short ret;
	struct {
		char number[51];
		short port;
		char file[PATH_MAX];
	} args;
	
	if (argc < 4)
		errx(1, "usage: %s phone port conf", argv[0]);

	strncpy(args.number, argv[1], 50);
	args.number[strlen(argv[1])] = '\0';

	args.port = atoi(argv[2]);

	strncpy(args.file, argv[3], PATH_MAX);
	args.file[PATH_MAX-1] = '\0';
	
	printf("Phone : %s\n", args.number);
	printf("Port  : %d\n", args.port);
	printf("Config: %s\n\n", args.file);

	dg_SetEventCallback(event_handler, &event_context);
	ret = dg_StartVoicerlib(NULL); 
	if (ret)
		err(1, "dg_StartVoicerLib(): %x", ret);

	/* catches up for clean exit */
	signal(SIGSTOP, sighandler);
	signal(SIGQUIT, sighandler);
	signal(SIGKILL, sighandler);
	signal(SIGINT, sighandler);
	
	ret = dg_CreateCallProgress(args.port, args.file);
	if (ret)
	{
	    if (ret==DG_WARNING_OLDFILEFORMAT)
	    {
		printf("ATENCAO: O arquivo %s parece estar danificado ou se trata de uma\n", args.file);
		printf("         versao muito antiga. Por favor, verifique se o arquivo\n");
		printf("         tem o terminador end=1 no final.\n");
	    }
	    else
		err(1, "dg_CreateCallProgress(): %x", ret);
	}
	ret = dg_EnableCallProgress(args.port, CP_ENABLE_LINETONE_OR_BUSY);
	if (ret)
	{
	    dg_ShutdownVoicerlib();
	    err(1, "dg_EnableCallProgress(): %x", ret);
	}
	ret = dg_PickUp(args.port, 1000);
	if (ret)
	{	
	    
    	    err(1, "dg_PickUp(): %x", ret);
	}
	else
	{
		
	}
	printf("Took port %u off-hook.\n", args.port);

	ret = dg_EnableAnswerDetection(args.port);
	if (ret)
		err(1, "dgEnableAnswerDetection(): %x", ret);

	while (!dtone);
	
	ret = dg_Dial(args.port, args.number, 1000, dtTone);
	if (ret)
		err(1, "dg_Dial(%s) : %x", args.number, ret);

	printf("Dialing %s...\n", args.number); 

	ret = dg_EnableAnswerDetection(args.port);
	if (ret)
		err(1, "dgEnableAnswerDetection(): %x", ret);

	while (!answered);
	
	ret = dg_HangUp(args.port);
	if (ret)
		err(1, "dg_HangUp(): %x", ret);
	printf("Putting port %u on-hook.\n", args.port);
	
	ret = dg_ShutdownVoicerlib();
	if (ret)
		err(ret, "dg_ShutdownVoicerLib()");

	return DG_EXIT_SUCCESS;
}
