/*---------------------------------------------------------------------------*\
 
    FILE....: dgr2loop.c
    AUTHOR..: Paulo Garcia
    DATE....: 13/07/2006
 
    Program to test R2 Loop

	Atenção: os sleeps neste programa só tem a finalizade de sincronizar
    as mensagens com os eventos. Só é possivel utiliza-lo porque estamos 
    tratando apenas um canal por vez

---------------------------------------------------------------------------*/
 
/*-------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#ifdef __LINUX__
	#include <strings.h>
	#include <pthread.h>
	#include <getopt.h>
#else
	#include <string.h>
	#include <conio.h>
	#include <windows.h>
	#define DllImport   __declspec(dllimport)
#endif


#include <vlibdef.h>
#include <voicerlib.h>

#define MAX_PATH 	255
#define MAX_CH		360

dg_event_data_structure event_context;

int rings=0;

void GetCardName(short nCard, char *szCardName)
{
	switch(dg_GetCardType(nCard))
	{
		case VBE13060PCI:
			strcpy(szCardName,"E1 30/60 rev.1\0");
			break;
		case VB0408PCI:
			strcpy(szCardName,"VoicerBox 0408\0");
			break;	
		case VBE13060PCI_R:
			strcpy(szCardName,"E1 30/60 rev.2\0");
			break;
		case VBE16060PCI:
			strcpy(szCardName,"E1 60/60 slim\0");
			break;
	}
}

void prompt(char *msg)
{
	printf("%s",msg);
	getchar();
	printf("\n");
}

void ReceiveEvents(void *context_data)
{

    dg_event_data_structure *EventContext;
    char data[255];
    char szDig[255];
    data[0] = 0;

    /* Copy received Data */
    EventContext = ((dg_event_data_structure*)context_data);

	//sprintf(data,"< cmd: %x data: %x >", EventContext->command, EventContext->data);
 
	switch (EventContext->command)
	{
		case EV_RINGS:
			rings++;
			sprintf(data,"      RING");
			if (rings>2)
			{
				//atende
				dg_PickUp(EventContext->port,100);
			}
			break;       
		case EV_DTMF:
			sprintf(data,"\t%c", EventContext->data);
			break;       
		case EV_BUSY:
		sprintf(data,"BUSY");
			break;
		case EV_CALLING:
			sprintf(data,"      CALLING");
			break;

		case EV_E1CHANGESTATUS:
			//this events are related only for E1 R2 protocol
			switch (EventContext->data)
			{
				case C_ID_RECEIVED:
					dg_GetCallerId(EventContext->port,szDig);
					sprintf(data,"CLID - %s",szDig);
					break;
				case C_ANSWERED:
					sprintf(data,"\t<<< Atendimento >>>");
					break;    
				/*case C_NUMBER_RECEIVED:
					dg_GetE1Number(EventContext->port, szDig);
					sprintf(data,"E1 Number = %s", szDig);
					break;*/
				case C_CONGESTION:
					sprintf(data,"E1 Events: Congestion");
					break;
				case C_B_ENDCALL:
					sprintf(data,"E1 Events: B Subscriber hangup");
					break;
				case C_NOTCOMPLETED:
					sprintf(data,"E1 Events: Call couldt be completed");
					break;
				case C_UNAVAILABLE:
					sprintf(data,"E1 Events: Channel unavailable");
					break;
			}
			break;
		case EV_R2:
		    sprintf(szDig,"%x", EventContext->data);
			sprintf(data,szDig);
			break;
	}
	if (strlen(data)>0)
	{
		if (EventContext->port>30)
			printf("\t\t\t");
	
		printf("%s\n",data);
	}
}

//------------------------------------------------------------------------------------------
// Main Function 
//------------------------------------------------------------------------------------------
int main (int argc, char *argv[])
{
    int ret, i;
    char szTemp[MAX_PATH];
    int ch_e1=9;

	printf("\ndgR2loop - Programa de teste para placas E1\n");
	printf("Digivoice Eletrônica Ltda\n\n");


	if (argc!=2)
	{
		printf("ERRO: Parametros invalidos!\n");
		printf("Utilize: dg2rloop <porta>\n");
		exit(1);
	}

	ch_e1 = atoi(argv[1]);
	printf("Canal a testar: %d\n\n",ch_e1);

	printf("\n-----------------------------------------------------------------------------\n");	
	printf("Este programa irá testar a placa E1 com 2 troncos em loop.\n");
	printf("Somente a primeira placa será testada, portanto certifique-se\n");
	printf("que a placa E1 seja a primeira a ser reconhecida. Para isso, o\n");
	printf("o utilitário dglist poderá ser utilizado.\n");
	printf("(*) Voce deverá conectar os 2 E1s em loop, ou seja o TX1->RX2 e TX2->RX1\n");
	printf("\n-----------------------------------------------------------------------------\n");	
	prompt("Tecle <ENTER> para continuar...\n");

	printf("ATENÇÃO: Certifique-se que nenhum outro programa que utiliza as\n");
	printf("placas DigiVoice esteja em execução!\n");
	prompt("Tecle <ENTER> para continuar...\n");

	dg_SetEventCallback(ReceiveEvents,&event_context);
	/* Start VoicerLib engine */
	printf("Iniciando Voicerlib...\n");
	sleep(2);
	if ((ret = dg_StartVoicerlib(NULL)) != DG_EXIT_SUCCESS)
	{
			printf("\nError starting voicerlib (code %d)\n",ret);
			
			switch(ret)
			{
				case 0x400:
					printf("Este erro pode ser causado por um dos seguintes motivos:\n");
					printf("\t- O modulo do kernel vlibd não foi carregado\n");
					printf("\t- Os inodes /dev/vlibd* nao existem\n");
					printf("\t- Falta de memória (o menos provavel)\n");
					break;
			}
			exit(EXIT_FAILURE);
	}
	printf("Voicerlib iniciada com sucesso!\n\n");

	printf("Sera efetuada uma discagem do canal %d para o canal %d\n", ch_e1, ch_e1+30);
	prompt("Tecle <ENTER> para continuar...\n");

	for (i=ch_e1;i<=(ch_e1+30); i+=30)
	{
		if (dg_GetPortInterface(i)==DG_DIGITAL_INTERFACE)
		{
			ret = dg_CreateE1Thread(i);
			if (ret != DG_EXIT_SUCCESS)
				printf("Error ao criar E1 Handle - porta %d\n",i);
			else
			{
				//numero qualquer para discagem
				strcpy(szTemp,"1141952557\0");
				dg_SetPortId(i,szTemp);
				//set the same category  for all channels
				dg_ConfigE1Thread(i,E1CFG_GROUP_II,1);
				//set to receive only 4 digits
				dg_ConfigE1Thread(i,E1CFG_MAXDIGITS_RX,4);
				//
				dg_ConfigE1Thread(i,E1CFG_SEND_ID_AFTERDIGIT,2);
				printf("Controle E1 criado - porta %d\n",i);
			}
		}
	}
	printf("Todas as threads de controle foram criadas.\n");
	sleep(1);
	prompt("Tecle <ENTER> para iniciar a discagem...\n");

	printf("------------------------------------------------------\n");
	printf("      %d\t\t\t%d\n",ch_e1,ch_e1+30);
	printf("------------------------------------------------------\n");
	printf("R2D     MFF            R2D      MFT       \n");
	printf("------------------------------------------------------\n");

	dg_PickUp(ch_e1,100);
	sleep(1);	//numa aplicação real não usar sleep!!!!!!!!!
	dg_Dial(ch_e1,"2558",500,dtDTMF);

	sleep(17);
	prompt("Tecle <ENTER> para desligar\n");
	printf("------------------------------------------------------\n");
	printf("      %d\t\t\t%d\n",ch_e1,ch_e1+30);
	printf("------------------------------------------------------\n");
	printf("R2D     MFF            R2D      MFT       \n");
	printf("------------------------------------------------------\n");
	dg_HangUp(ch_e1);
	dg_HangUp(ch_e1+30);
	sleep(2);
	prompt("\n\nTecle <ENTER> para finalizar\n");
	ret = dg_DestroyE1Thread(ch_e1);
	ret = dg_DestroyE1Thread(ch_e1+30);
	dg_ShutdownVoicerlib();
	printf("\n\n* * * Goodbye! * * *\n\n");
	return DG_EXIT_SUCCESS;
}

