/*---------------------------------------------------------------------------*\
 
    FILE....: simple.c
    AUTHOR..: Paulo Garcia
    DATE....: 27/02/2004
 
    Very simple sample, starting and shutting down
 
    This program requires ncurses-devel

---------------------------------------------------------------------------*/
 
/*-------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#ifdef __LINUX__
	#include <strings.h>
	#include <ncurses.h>
	#include <pthread.h>
	#include <getopt.h>
#else
	#include <string.h>
	#include <conio.h>
	#include <windows.h>
	#define DllImport   __declspec(dllimport)
#endif


#include "../../../src_common/voicerlib.h"
#include "../../../src_common/vlibdef.h"

#define MAX_PATH 	255
#define MAX_CH		360

WINDOW *win;
extern pthread_mutex_t event_mutex;
WINDOW *win_ev;	//windows events

void destroy_win(WINDOW *local_win);

unsigned short current_channel;


//Global flags to hold channels states
short bPickedUp[MAX_CH];
short bAlarmOn[MAX_CH];
short bRecOn[MAX_CH];
short bE1[MAX_CH];


//-------------------------------------------------------------------
// Functions to manage ncurses windows
//-------------------------------------------------------------------
WINDOW *create_newwin(int height, int width, int starty, int startx)
{
	WINDOW *local_win;

	local_win = newwin(height, width, starty, startx);
	wrefresh(local_win);		/* Show that box 		*/

	return local_win;
}

void destroy_win(WINDOW *local_win)
{	
	/* box(local_win, ' ', ' '); : This won't produce the desired
	 * result of erasing the window. It will leave it's four corners 
	 * and so an ugly remnant of window. 
	 */
	wborder(local_win, ' ', ' ', ' ',' ',' ',' ',' ',' ');
	/* The parameters taken are 
	 * 1. win: the window on which to operate
	 * 2. ls: character to be used for the left side of the window 
	 * 3. rs: character to be used for the right side of the window 
	 * 4. ts: character to be used for the top side of the window 
	 * 5. bs: character to be used for the bottom side of the window 
	 * 6. tl: character to be used for the top left corner of the window 
	 * 7. tr: character to be used for the top right corner of the window 
	 * 8. bl: character to be used for the bottom left corner of the window 
	 * 9. br: character to be used for the bottom right corner of the window
	 */
	wrefresh(local_win);
	delwin(local_win);
}


dg_event_data_structure event_context;

//---------- NCURSES screen functions -----------

void print_event(char *szMsg, int port)
{
	
	struct tm *now;
	time_t curSecs;
	curSecs = time(NULL);				
	now = localtime(&curSecs);
	
	attron(COLOR_PAIR(2));
	wprintw(win_ev,"<%02d:%02d:%02d> port %d: %s\n",now->tm_hour, now->tm_min, now->tm_sec, port,szMsg);
	attroff(COLOR_PAIR(2));
	wrefresh(win_ev);
	
}

/* Print Main Screen */
void print_screen()
{
    /* Print Main Screen */
    attron(COLOR_PAIR(1));
    box(win,'|', '-');    
    mvprintw(0,30,"   Digivoice VoicerLib Sample   ");
    refresh();
    attroff(COLOR_PAIR(1));

    mvprintw(1,1,"Placas: %d", dg_GetCardsCount());
    attron(COLOR_PAIR(3));
    /* Print Menu */
    mvprintw(3,1," <q>uit            <p>pickup/hangup   <r>record start/stop");
    mvprintw(4,1," <a>alarm on/off   <x>r2 on/off       </> E1 Thread on/off ");
    attroff(COLOR_PAIR(3));
}
//---------- end of NCURSES screen functions -----------

//------------------------------------------------------------------------------------------
// Function that will be receive events from library 
//------------------------------------------------------------------------------------------
void ReceiveEvents(void *context_data)
{
    dg_event_data_structure *EventContext;
    char data[255];
    char szDig[MAX_PATH];
    
    data[0] = 0;

    /* Copy received Data */
    EventContext = ((dg_event_data_structure*)context_data);

		sprintf(data,"cmd %02x - data %02x",EventContext->command, EventContext->data);
		print_event(data , EventContext->port);
		switch(EventContext->command)
		{
			case EV_RINGS:
				print_event("RING" , EventContext->port);
				break;
			case EV_R2:
				sprintf(data,"Received R2 = %02x",EventContext->data);
				print_event(data , EventContext->port);
				break;
			case EV_CALLERID:
				dg_GetCallerId(EventContext->port,szDig);
				sprintf(data,"CallerID: %s", szDig);
				print_event(data , EventContext->port);
				break;
				
		}
 
}

//----------------------------------------------------------
//Shows help messages
//----------------------------------------------------------
void show_help()
{
		printf("\nInvalid Parameters!");
		printf("\nUsage: simple -p <firmware_path>");
		printf("\n       Where: firm_path - Path to the firmware path     \n");
}

//------------------------------------------------------------------------------------------
// Main Function 
//------------------------------------------------------------------------------------------
int main (int argc, char *argv[])
{
    char ch;
    char szNumber[2];
    char szTemp[255];
    char szMsg[255];
    const char* program_name;
    int next_option,ret;
    int i,card_count;
    short paused=0;
  
    WINDOW *wndH100;    
    if (argc < 2)
    {
    	show_help();
    	exit(EXIT_FAILURE);
    }
	
	/* Parse Command line arguments */
	/* parameters: no_cards , card_type, szFirmwarePath */ 
	const char* const short_options = "c:t:p:";
	
	const struct option long_options[] = {
		{ "path", 1, NULL, 'p'}, 		
		{ NULL, 0, NULL, 0 }
	};
		
	const char output_filename = (const char)NULL;

	//-------------------------------------------------------------------
	// command line parser
	//-------------------------------------------------------------------
	program_name = argv[0]; /* Saves program name*/
		
	do {
		next_option = getopt_long(argc, argv, short_options, long_options, NULL);
		
		switch(next_option)
		{
			case 'p':		/* -p or --path */
				sscanf(optarg,"%s",&szTemp);				
				if (strlen(szTemp)==0)
				{
						show_help();
						exit(EXIT_FAILURE);
				}
				break;
		}
	}
	while (next_option != -1);
	//-------------------------------------------------------------------
		
	//path to firmware	
	printf("Locating firmware at %s...\n",szTemp);
	/* Set events function callback */
	dg_SetEventCallback(ReceiveEvents,&event_context);
	digivoice_sleep(2);

	/* Start VoicerLib engine */
	if ((ret = dg_StartVoicerlib(szTemp)) != DG_EXIT_SUCCESS)
	{
			printf("\nError starting voicerlib (code %d)\n",ret);
			exit(EXIT_FAILURE);
	}

	//Initializes global flags
	for (i=0;i< dg_GetPortsCount(); i++)
	{
		bPickedUp[i] = 0;
		bAlarmOn[i] = 0;
		bRecOn[i] = 0;
		bE1[i] = 0;
	}


	//sync mode for all cards
	for (i=0;i< dg_GetCardsCount(); i++)
	{
		dg_SetCardSyncMode(i+1,MASTER_INTERNAL_SYNC);
	}

	printf("Stating GUI....\n",szTemp);	
	/* initialize ncurses stuff */
	win = initscr();
	start_color();  /* Start the color functionality */
	cbreak();       /* Line buffering disabled, Pass on
										* every thing to me         */
	keypad(stdscr, TRUE);        /* I need that nifty F1     */
	noecho();
	
	/* Initialize used colors */
	init_pair(1, COLOR_CYAN, COLOR_BLACK);        
	init_pair(2, COLOR_RED, COLOR_WHITE);            
	init_pair(3, COLOR_GREEN, COLOR_BLACK);            

	/* print initial screen */
	print_screen();
	
	/* disable getch wait */
	nodelay(win, TRUE);


	//create event window
	win_ev = create_newwin(12,78,11,1);
	wbkgd(win_ev,A_REVERSE | A_BOLD);
	scrollok(win_ev, TRUE);
	wrefresh(win_ev);

	current_channel = 1;	
	/* Main Loop */
	while((ch = getch()) != 'q')
	{
		if (ch == 'Q') 
		{
			print_event("Quit...",1);
			break;
		}

		switch(ch) 
		{
			case '-':
				if (current_channel > 1)
						current_channel--;
				print_event("Current Channel: ",current_channel);
				break;
			case '+':
				if (current_channel < dg_GetPortsCount())
						current_channel++;               
				print_event("Current Channel: ",current_channel);
				break;
			case 'x': case 'X':
				dg_SendR2Command(1, R2_ENABLE);
				dg_SendR2Command(31, R2_ENABLE);
				print_event("Enabling R2", current_channel);
				break;
			case 'r': case 'R':
				if (bRecOn[current_channel-1])
				{
					dg_StopRecordFile(current_channel);
					print_event("RecOff Off!", current_channel);
					bRecOn[current_channel-1] = 0;
				}
				else
				{
					dg_SetRecordFormat(current_channel,ffWaveULaw);
					sprintf(szTemp,"/tmp/grava%d.wav",current_channel);
					dg_RecordFile(current_channel, szTemp,"");
					print_event("RecOff On - Wave!", current_channel);
					bRecOn[current_channel-1] = 1;
				}
				break;
			case 'a': case 'A':		//enable/disable alarm
				if (bAlarmOn[current_channel-1])
				{
					for (i=0;i< dg_GetCardsCount(); i++)					
					{
						dg_SetAlarmMode(i+1,ALARM_MANUAL_NOTIFY);
						print_event("Alarm Off!", i+1);
					}
				}
				else
				{
					for (i=0;i< dg_GetCardsCount(); i++)					
					{
						dg_SetAlarmMode(i+1,ALARM_AUTOMATIC_NOTIFY);
						print_event("Alarm On!", i+1);
					}
				}
				//invert state
				bAlarmOn[current_channel-1] = !bAlarmOn[current_channel-1];
				break;
			case 'd': case 'D':
				dg_Dial(current_channel,"2345678",2000,dtDTMF);
				break;
			case 'p': case 'P':		//pickup/hangup
				if (bPickedUp[current_channel-1])
				{
					dg_HangUp(current_channel);
					print_event(" < HangUp!", current_channel);
				}
				else
				{
					dg_PickUp(current_channel,100);
					print_event(" > PickUp!", current_channel);
				}
				//invert state
				bPickedUp[current_channel-1] = !bPickedUp[current_channel-1];
				break;
			case '/':		//e1 on/off
				if (bE1[current_channel-1])
				{
					dg_DestroyE1Thread(current_channel);
					print_event(" < E1 Thread Off!", current_channel);
				}
				else
				{
					dg_CreateE1Thread(current_channel);
					print_event(" > E1 Thread On!", current_channel);
					
					//set the Port id for all channels
					strcpy(szTemp,"1141952557\0");
					dg_SetPortId(current_channel,szTemp);
					//set the same category  for all channels
					dg_ConfigE1Thread(current_channel,E1CFG_GROUP_II,1);
					//set to receive only 4 digits
					dg_ConfigE1Thread(current_channel,E1CFG_MAXDIGITS_RX,4);
					
				}
				//invert state
				bE1[current_channel-1] = !bE1[current_channel-1];
				break;
				
		}	//switch
  	} /* while */

	destroy_win(win_ev);
	endwin();
	/* Stops voicerlib engine */
	dg_ShutdownVoicerlib();
	printf("\n\n* * * Goodbye! * * *\n\n");
	return DG_EXIT_SUCCESS;
}

